@@ -68,13 +68,13 @@ typedef struct _CtkDisplayConfig
     GtkWidget *chk_xinerama_enabled;
     GtkWidget *chk_primary_display;
 
-    GtkWidget *notebook; /* Tabbed notebook for display and X screen pages */
-
 
     /* Display - Info */
     GtkWidget *display_page;
 
-    GtkWidget *txt_display_model;
+    GtkWidget *mnu_display_model;
+    nvDisplayPtr *display_model_table; /* Lookup table for display modelname */
+    int display_model_table_len;
     GtkWidget *txt_display_gpu;
 
     GtkWidget *btn_display_config;
