@@ -431,7 +431,8 @@ void ctk_display_layout_set_display_position (CtkDisplayLayout *ctk_object,
 void ctk_display_layout_set_display_panning (CtkDisplayLayout *ctk_object,
                                              nvDisplayPtr display,
                                              int width, int height);
-
+void ctk_display_layout_select_display (CtkDisplayLayout *ctk_object,
+                                        nvDisplayPtr display);
 void ctk_display_layout_update_display_count (CtkDisplayLayout *,
                                               nvDisplayPtr);
 
