@@ -40,7 +40,6 @@
 
 #include "ctkevent.h"
 #include "NVCtrlLib.h"
-#include "msg.h"
 
 static void ctk_event_class_init(CtkEventClass *ctk_event_class);
 
@@ -224,7 +223,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_PROBE_DISPLAYS);
     MAKE_SIGNAL(NV_CTRL_REFRESH_RATE);
     MAKE_SIGNAL(NV_CTRL_INITIAL_PIXMAP_PLACEMENT);
-    MAKE_SIGNAL(NV_CTRL_GLYPH_CACHE);
     MAKE_SIGNAL(NV_CTRL_PCI_BUS);
     MAKE_SIGNAL(NV_CTRL_PCI_DEVICE);
     MAKE_SIGNAL(NV_CTRL_PCI_FUNCTION);
@@ -245,14 +243,8 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_FRAMELOCK_SYNC_RATE_4);
     MAKE_SIGNAL(NV_CTRL_GVO_LOCK_OWNER);
     MAKE_SIGNAL(NV_CTRL_NUM_GPU_ERRORS_RECOVERED);
-    MAKE_SIGNAL(NV_CTRL_REFRESH_RATE_3);
-    MAKE_SIGNAL(NV_CTRL_GVO_OUTPUT_VIDEO_LOCKED);
-    MAKE_SIGNAL(NV_CTRL_GVO_SYNC_LOCK_STATUS);
-    MAKE_SIGNAL(NV_CTRL_GVO_ANC_TIME_CODE_GENERATION);
     MAKE_SIGNAL(NV_CTRL_ONDEMAND_VBLANK_INTERRUPTS);
-    MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE);
-    MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_ALPHA_KEY);
-    MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_NUM_KEY_RANGES);
+    MAKE_SIGNAL(NV_CTRL_NOTEBOOK_DISPLAY_CHANGE_LID_EVENT);
 
 #undef MAKE_SIGNAL
     
@@ -263,7 +255,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GVO_COMPOSITE_NUM_KEY_RANGES
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_NOTEBOOK_INTERNAL_LCD
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -477,9 +469,7 @@ static gboolean ctk_event_dispatch(GSource *source,
      * Handle the ATTRIBUTE_CHANGED_EVENT NV-CONTROL event
      */
 
-    if (event_source->event_base != -1 &&
-        (event.type == (event_source->event_base + ATTRIBUTE_CHANGED_EVENT))) {
-
+    if (event.type == (event_source->event_base + ATTRIBUTE_CHANGED_EVENT)) {
         XNVCtrlAttributeChangedEvent *nvctrlevent =
             (XNVCtrlAttributeChangedEvent *) &event;
 
@@ -504,15 +494,14 @@ static gboolean ctk_event_dispatch(GSource *source,
                                 NV_CTRL_TARGET_TYPE_X_SCREEN,
                                 nvctrlevent->screen);
         }
+    }
 
     /* 
      * Handle the TARGET_ATTRIBUTE_CHANGED_EVENT NV-CONTROL event
      */
 
-    } else if (event_source->event_base != -1 &&
-               (event.type == (event_source->event_base
-                               +TARGET_ATTRIBUTE_CHANGED_EVENT))) {
-
+    if (event.type == (event_source->event_base
+                       +TARGET_ATTRIBUTE_CHANGED_EVENT)) {
         XNVCtrlAttributeChangedEventTarget *nvctrlevent =
             (XNVCtrlAttributeChangedEventTarget *) &event;
 
@@ -537,15 +526,14 @@ static gboolean ctk_event_dispatch(GSource *source,
                                 nvctrlevent->target_type,
                                 nvctrlevent->target_id);
         }
+    }
 
     /*
      * Also handle XRandR events.
      */
-
-    } else if (event_source->randr_event_base != -1 &&
-               (event.type ==
-                (event_source->randr_event_base + RRScreenChangeNotify))) {
-        
+    if (event_source->randr_event_base != -1 &&
+        event.type ==
+        (event_source->randr_event_base + RRScreenChangeNotify)) {
         XRRScreenChangeNotifyEvent *xrandrevent =
             (XRRScreenChangeNotifyEvent *)&event;
         int screen;
@@ -559,14 +547,6 @@ static gboolean ctk_event_dispatch(GSource *source,
                                 NV_CTRL_TARGET_TYPE_X_SCREEN,
                                 screen);
         }
-
-    /*
-     * Trap events that get registered but are not handled
-     * properly.
-     */
-
-    } else {
-        nv_warning_msg("Unknown event type %d.", event.type);
     }
     
     return TRUE;
