@@ -60,9 +60,8 @@ static char *nv_strndup(char *s, int n);
 #define A NV_PARSER_TYPE_NO_QUERY_ALL
 #define Z NV_PARSER_TYPE_NO_ZERO_VALUE
 #define H NV_PARSER_TYPE_100Hz
-#define K NV_PARSER_TYPE_1000Hz
 #define S NV_PARSER_TYPE_STRING_ATTRIBUTE
-#define I NV_PARSER_TYPE_SDI
+#define W NV_PARSER_TYPE_VALUE_IS_SWITCH_DISPLAY
 
 AttributeTableEntry attributeTable[] = {
    
@@ -88,8 +87,11 @@ AttributeTableEntry attributeTable[] = {
     { "HWOverlay",             NV_CTRL_HWOVERLAY,                   0     },
     { "Stereo",                NV_CTRL_STEREO,                      0     },
     { "TwinView",              NV_CTRL_TWINVIEW,                    0     },
-    { "ConnectedDisplays",     NV_CTRL_CONNECTED_DISPLAYS,          0     },
-    { "EnabledDisplays",       NV_CTRL_ENABLED_DISPLAYS,            0     },
+    { "ConnectedDisplays",     NV_CTRL_CONNECTED_DISPLAYS,          D     },
+    { "EnabledDisplays",       NV_CTRL_ENABLED_DISPLAYS,            D     },
+    { "NotebookInternalLCD",   NV_CTRL_NOTEBOOK_INTERNAL_LCD,       N|D   },
+    { "NotebookDisplayChangeLidEvent", NV_CTRL_NOTEBOOK_DISPLAY_CHANGE_LID_EVENT, N },
+    { "SwitchToDisplays",      NV_CTRL_SWITCH_TO_DISPLAYS,          D|N|W },
     { "AssociatedDisplays",    NV_CTRL_ASSOCIATED_DISPLAY_DEVICES,  N|D   },
     { "ProbeDisplays",         NV_CTRL_PROBE_DISPLAYS,              A     },
     { "ForceGenericCpu",       NV_CTRL_FORCE_GENERIC_CPU,           0     },
@@ -105,9 +107,7 @@ AttributeTableEntry attributeTable[] = {
     { "FSAAAppControlled",     NV_CTRL_FSAA_APPLICATION_CONTROLLED, 0     },
     { "LogAnisoAppControlled", NV_CTRL_LOG_ANISO_APPLICATION_CONTROLLED,0 },
     { "RefreshRate",           NV_CTRL_REFRESH_RATE,                N|H   },
-    { "RefreshRate3",          NV_CTRL_REFRESH_RATE_3,              N|K   },
     { "InitialPixmapPlacement",NV_CTRL_INITIAL_PIXMAP_PLACEMENT,    N     },
-    { "GlyphCache",            NV_CTRL_GLYPH_CACHE,                 N     },
     { "PCIBus",                NV_CTRL_PCI_BUS,                     N     },
     { "PCIDevice",             NV_CTRL_PCI_DEVICE,                  N     },
     { "PCIFunc",               NV_CTRL_PCI_FUNCTION,                N     },
@@ -146,34 +146,28 @@ AttributeTableEntry attributeTable[] = {
     { "FrameLockMasterable",   NV_CTRL_FRAMELOCK_MASTERABLE,        N|F|G },
     { "FrameLockFPGARevision", NV_CTRL_FRAMELOCK_FPGA_REVISION,     N|F|G },
 
-    { "GvoSupported",                    NV_CTRL_GVO_SUPPORTED,                        I|N },
-    { "GvoSyncMode",                     NV_CTRL_GVO_SYNC_MODE,                        I },
-    { "GvoSyncSource",                   NV_CTRL_GVO_SYNC_SOURCE,                      I },
-    { "GvoOutputVideoFormat",            NV_CTRL_GVO_OUTPUT_VIDEO_FORMAT,              I },
-    { "GvoInputVideoFormat",             NV_CTRL_GVO_INPUT_VIDEO_FORMAT,               I|N },
-    { "GvoDataFormat",                   NV_CTRL_GVO_DATA_FORMAT,                      I },
-    { "GvoDisplayXScreen",               NV_CTRL_GVO_DISPLAY_X_SCREEN,                 I|N },
-    { "GvoCompositeSyncInputDetected",   NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECTED,    I|N },
-    { "GvoCompositeSyncInputDetectMode", NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECT_MODE, I|N },
-    { "GvoSdiSyncInputDetected",         NV_CTRL_GVO_SDI_SYNC_INPUT_DETECTED,          I|N },
-    { "GvoVideoOutputs",                 NV_CTRL_GVO_VIDEO_OUTPUTS,                    I|N },
-    { "GvoSyncDelayPixels",              NV_CTRL_GVO_SYNC_DELAY_PIXELS,                I },
-    { "GvoSyncDelayLines",               NV_CTRL_GVO_SYNC_DELAY_LINES,                 I },
-    { "GvoGlxLocked",                    NV_CTRL_GVO_GLX_LOCKED,                       I|N },
-    { "GvoXScreenPanX",                  NV_CTRL_GVO_X_SCREEN_PAN_X,                   I },
-    { "GvoXScreenPanY",                  NV_CTRL_GVO_X_SCREEN_PAN_Y,                   I },
-    { "GvoOverrideHwCsc",                NV_CTRL_GVO_OVERRIDE_HW_CSC,                  I },
-    { "GvoCapabilities",                 NV_CTRL_GVO_CAPABILITIES,                     I|N },
-    { "GvoCompositeTermination",         NV_CTRL_GVO_COMPOSITE_TERMINATION,            I },
-    { "GvoFlipQueueSize",                NV_CTRL_GVO_FLIP_QUEUE_SIZE,                  I },
-    { "GvoFirmwareVersion",              NV_CTRL_STRING_GVO_FIRMWARE_VERSION,          I|S|N },
-    { "GvoLockOwner",                    NV_CTRL_GVO_LOCK_OWNER,                       I|N },
-    { "GvoOutputVideoLocked",            NV_CTRL_GVO_OUTPUT_VIDEO_LOCKED,              I|N },
-    { "GvoSyncLockStatus",               NV_CTRL_GVO_SYNC_LOCK_STATUS,                 I|N },
-    { "GvoANCTimeCodeGeneration",        NV_CTRL_GVO_ANC_TIME_CODE_GENERATION,         I },
-    { "GvoComposite",                    NV_CTRL_GVO_COMPOSITE,                        I },
-    { "GvoCompositeAlphaKey",            NV_CTRL_GVO_COMPOSITE_ALPHA_KEY,              I },
-    { "GvoCompositeNumKeyRanges",        NV_CTRL_GVO_COMPOSITE_NUM_KEY_RANGES,         I|N },
+    { "GvoSupported",                    NV_CTRL_GVO_SUPPORTED,                        N },
+    { "GvoSyncMode",                     NV_CTRL_GVO_SYNC_MODE,                        N },
+    { "GvoSyncSource",                   NV_CTRL_GVO_SYNC_SOURCE,                      N },
+    { "GvoOutputVideoFormat",            NV_CTRL_GVO_OUTPUT_VIDEO_FORMAT,              N },
+    { "GvoInputVideoFormat",             NV_CTRL_GVO_INPUT_VIDEO_FORMAT,               N },
+    { "GvoDataFormat",                   NV_CTRL_GVO_DATA_FORMAT,                      N },
+    { "GvoDisplayXScreen",               NV_CTRL_GVO_DISPLAY_X_SCREEN,                 N },
+    { "GvoCompositeSyncInputDetected",   NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECTED,    N },
+    { "GvoCompositeSyncInputDetectMode", NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECT_MODE, N },
+    { "GvoSdiSyncInputDetected",         NV_CTRL_GVO_SDI_SYNC_INPUT_DETECTED,          N },
+    { "GvoVideoOutputs",                 NV_CTRL_GVO_VIDEO_OUTPUTS,                    N },
+    { "GvoSyncDelayPixels",              NV_CTRL_GVO_SYNC_DELAY_PIXELS,                N },
+    { "GvoSyncDelayLines",               NV_CTRL_GVO_SYNC_DELAY_LINES,                 N },
+    { "GvoGlxLocked",                    NV_CTRL_GVO_GLX_LOCKED,                       N },
+    { "GvoXScreenPanX",                  NV_CTRL_GVO_X_SCREEN_PAN_X,                   N },
+    { "GvoXScreenPanY",                  NV_CTRL_GVO_X_SCREEN_PAN_Y,                   N },
+    { "GvoOverrideHwCsc",                NV_CTRL_GVO_OVERRIDE_HW_CSC,                  N },
+    { "GvoCapabilities",                 NV_CTRL_GVO_CAPABILITIES,                     N },
+    { "GvoCompositeTermination",         NV_CTRL_GVO_COMPOSITE_TERMINATION,            N },
+    { "GvoFlipQueueSize",                NV_CTRL_GVO_FLIP_QUEUE_SIZE,                  N },
+    { "GvoFirmwareVersion",              NV_CTRL_STRING_GVO_FIRMWARE_VERSION,          S|N },
+    { "GvoLockOwner",                    NV_CTRL_GVO_LOCK_OWNER,                       N },
 
     { "Brightness",            BRIGHTNESS_VALUE|ALL_CHANNELS,       N|C|G },
     { "RedBrightness",         BRIGHTNESS_VALUE|RED_CHANNEL,        C|G   },
@@ -223,7 +217,10 @@ AttributeTableEntry attributeTable[] = {
     { "XF86VidModeVersion",     NV_CTRL_STRING_XF86VIDMODE_VERSION,      S|N },
     { "XvVersion",              NV_CTRL_STRING_XV_VERSION,               S|N },
     { "SLIMode",                NV_CTRL_STRING_SLI_MODE,                 S|N },
-    
+    { "GPUCurrentPerfMode",     NV_CTRL_GPU_CURRENT_PERFORMANCE_MODE,    N },
+    { "GPUCurrentPerfLevel",    NV_CTRL_GPU_CURRENT_PERFORMANCE_LEVEL,   N },
+    { "GPUAdaptiveClockState",  NV_CTRL_GPU_ADAPTIVE_CLOCK_STATE,        N },
+    { "GPUPowerSource",         NV_CTRL_GPU_POWER_SOURCE,                N },
     { NULL,                    0,                                   0     }
 };
 
@@ -237,9 +234,8 @@ AttributeTableEntry attributeTable[] = {
 #undef A
 #undef Z
 #undef H
-#undef K
 #undef S
-#undef I
+#undef W
 
 /*
  * When new integer attributes are added to NVCtrl.h, an entry should
@@ -248,7 +244,7 @@ AttributeTableEntry attributeTable[] = {
  * about.
  */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GVO_COMPOSITE_NUM_KEY_RANGES
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_NOTEBOOK_INTERNAL_LCD
 #warning "Have you forgotten to add a new integer attribute to attributeTable?"
 #endif
 
